#!/usr/bin/python
# -*- coding: utf-8 -*-

import pygame
from pygame.locals import *
from random import choice, shuffle
import time
import textwrap
import sys
import os

#something for maemo, taken from twaeltis's pypackager wiki
os.environ["SDL_VIDEO_X11_WMCLASS"]="reactionfaceoff"

from minigames import (crashgame, colorgame, calcgame,
                      sizegame, colormatchgame, sadsmileygame,
                      isprimegame, countdowngame, minsymbolsgame,
                      singlesymbolgame, alphabetgame, flaggame,
                      fruitmathsgame)

class ReactionFaceoff(object):
    '''Main program'''
    def __init__(self):
        pygame.init()
        pygame.display.set_caption("Reaction face-off")
        try:
            if sys.argv[1] == "f":
                #~ self.display = pygame.display.set_mode((800,480),
                    #~ pygame.FULLSCREEN|pygame.DOUBLEBUF|pygame.HWSURFACE)
                self.screen = pygame.display.set_mode((800,480),
                    pygame.FULLSCREEN|pygame.DOUBLEBUF|pygame.HWSURFACE)
            else:
                #~ self.display = pygame.display.set_mode((800,480))
                self.screen = pygame.display.set_mode((800,480))
        except:
            #~ self.display = pygame.display.set_mode((800,480), pygame.RESIZABLE)
            self.screen = pygame.display.set_mode((800,480), pygame.RESIZABLE)
        #~ self.screen = pygame.Surface((800, 480))
        #~ self.screen.fill((0,0,0))
        pygame.mouse.set_visible(False)
        h = self.screen.get_height()
        w = self.screen.get_width()
        self.banner, r = self.load_image("main.png", -1)
        self.banner = pygame.transform.rotozoom(self.banner, 90,0.5)
        #~ self.starfield = Starfield(self, 150,0,w-150-150-100-100,h, 200, "up")
        # buttons for both players
        self.button1 = Button(self, 0, 0, 150, h, "left")
        self.button2 = Button(self, w-150, 0, 150, h, "right")
        # option buttons and sliders for before the game
        self.showIntrosB = Button(self, w-160-100-90, 0, 90, h, "right")
        self.showIntrosB.setCaption("Show tasks before games")
        self.showIntrosB.setColor((0, 71, 171))
        self.showIntrosB.toggle()
        ypos = -(4000 - self.screen.get_height())
        self.numberOfGamesS = Slider(self, w-160-90, ypos+300, 90, 4000, "right")
        self.numberOfGamesS.setCaption("Number of rounds")
        self.numberOfGamesS.setColor((0, 71, 171))
        self.quitB = Button(self, 160, 0, 140, 60, "down")
        self.quitB.setCaption("Quit")
        self.gamesB = Button(self, 160+150, 0, 130, 60, "down")
        self.gamesB.setCaption("Games")
        self.helpB = Button(self, 160+150, h-60, 130, 60, "up")
        self.helpB.setCaption("Help")
        self.aboutB = Button(self, 160, h-60, 140, 60, "up")
        self.aboutB.setCaption("About")
        # every area covers half the screen
        self.area1 = Rect(0, 0, w/2, h)
        self.area2 = Rect(w/2, 0, w/2, h)
        self.undesiredGames = []
        self.lastEvent = False
        # available games
        self.minigames=["Background Color", "Colliding Gems", "Continous equation",
               "Gem Sizes", "Color matches Name", "Everyone is happy",
               "Prime numbers", "Countdown to zero", "4 different gems",
               "Single different gem", "Order of alphabet", "Nation flags",
               "Fruit maths"]
        # main loop
        self.loop()
        pygame.quit()
    
    def getMiniGames(self, number):
        '''returns a list with games evenly distributed'''
        gamesTemp = self.minigames
        games = []
        for i in gamesTemp:
            if i not in self.undesiredGames:
                games.append(i)
        l = []
        counter = 0
        done = False
        while not done:
            shuffle(games)
            for i in games:
                l.append(i)
                counter += 1
                if counter == number:
                    done = True
                    break
        return l
    
    def getGameWithName(self, game):
        '''returns a new minigame with the given name'''
        #~ return singlesymbolgame.SingleSymbolGame(self)
        if game == "Background Color":
            return colorgame.ColorGame(self)
        if game == "Colliding Gems":
            return crashgame.CrashGame(self)
        if game == "Continous equation":
            return calcgame.CalcGame(self)
        if game == "Gem Sizes":
            return sizegame.SizeGame(self)
        if game == "Color matches Name":
            return colormatchgame.ColorMatchGame(self)
        if game == "Everyone is happy":
            return sadsmileygame.SadSmileyGame(self)
        if game == "Prime numbers":
            return isprimegame.IsPrimeGame(self)
        if game == "Countdown to zero":
            return countdowngame.CountdownGame(self)
        if game == "4 different gems":
            return minsymbolsgame.MinSymbolsGame(self)
        if game == "Single different gem":
            return singlesymbolgame.SingleSymbolGame(self)
        if game == "Order of alphabet":
            return alphabetgame.AlphabetGame(self)
        if game == "Nation flags":
            return flaggame.FlagGame(self)
        if game == "Fruit maths":
            return fruitmathsgame.FruitMathsGame(self)

    def loop(self):
        '''main loop'''
        clock = pygame.time.Clock()
        exitGame = False
        # main loop
        while not exitGame:
            # greeting phase
            self.greeting = True
            self.aboutScreen = False
            self.helpScreen = False
            self.gamesScreen = False
            self.pauseScreen = False
            self.ready1 = False
            self.ready2 = False
            self.playing = False
            self.button1.setColor((0, 71, 171))
            self.button2.setColor((0, 71, 171))
            #~ self.button2.setColor((100,100,100))
            self.button1.caption = False
            self.button2.caption = False
            while not (self.ready1 and self.ready2):
                exitGame = self.handleEvents()
                if exitGame == True: break
                self.clearPlayArea()
                hc = self.screen.get_height()/2
                self.screen.blit(self.banner, (160,hc-(self.banner.get_height()/2)))
                self.drawText("Reaction face-off", (400, hc),
                              self.screen.get_height()-40, "right", color = (255,255,255), size=40)
                self.showIntrosB.draw()
                #~ self.starfield.update()
                self.numberOfGamesS.draw()
                self.numberOfGamesS.slide()
                self.button1.draw()
                self.button2.draw()
                self.quitB.draw()
                self.gamesB.draw()
                self.helpB.draw()
                self.aboutB.draw()
                if not self.ready1:
                    self.button1.glow()
                if not self.ready2:
                    self.button2.glow()
                #~ self.screen = pygame.transform.rotate(self.screen, -90)
                #~ self.display.blit(self.screen, (0,0))
                pygame.display.flip()
                clock.tick(30)
                #~ exit()
            if exitGame: break
            # playing phase
            games = self.getMiniGames(self.numberOfGamesS.value)
            self.greeting = False
            self.points1 = 0
            self.points2 = 0
            returnToMenu = False
            for g in games:
                if returnToMenu: break
                self.clearPlayArea()
                self.g = self.getGameWithName(g)
                self.button1.setColor((0, 71, 171))
                self.button2.setColor((0, 71, 171))
                self.button1.caption = False
                self.button2.caption = False
                # maybe show an intro first
                if self.showIntrosB.toggled:
                    self.intro(self.g.task)
                self.button1.setCaption(self.g.task, (255,255,255), 40)
                self.button2.setCaption(self.g.task, (255,255,255), 40)
                self.button1.draw()
                self.button2.draw()
                self.clearPlayArea()
                endRound = False
                self.discardEvents()
                self.playing = True
                # action phase where stuff happens on screen
                while not endRound:
                    self.g.play()
                    self.drawScore()
                    status = self.handleEvents()
                    if (status == 1) or (status == 2):
                        self.playing = False
                        if self.lastEvent:
                            status = self.handleEvents()
                        if status == 1:
                            self.button1.setColor((0,255,0))
                            self.button1.setCaption(
                                         self.g.winMessage, (0,0,0), 60)
                            self.button2.setColor((255,0,0))
                            self.button2.setCaption(
                                         self.g.loseMessage, (0,0,0), 60)
                            self.points1 = self.points1 + 1
                        if status == 2:
                            self.button2.setColor((0,255,0))
                            self.button2.setCaption(
                                         self.g.winMessage, (0,0,0), 60)
                            self.button1.setColor((255,0,0))
                            self.button1.setCaption(
                                         self.g.loseMessage, (0,0,0), 60)
                            self.points2 = self.points2 + 1
                        self.button1.draw()
                        self.button2.draw()
                        #~ self.display.blit(self.screen, (0,0))
                        pygame.display.flip()
                        break
                    elif status == "return":
                        returnToMenu = True
                        break
                    elif status == "pause":
                        self.pauseScreen = True
                        self.pause()
                    self.button1.draw()
                    self.button2.draw()
                    #~ self.display.blit(self.screen, (0,0))
                    pygame.display.flip()
                    clock.tick(30)
                    if exitGame: break
                # Pause between games
                for i in range(0,60):
                    clock.tick(30)
            self.clearPlayArea()
            self.finalizeGame()
    
    def pause(self):
        self.clearPlayArea()
        self.drawMainText("Pause")
        pygame.display.flip()
        clock = pygame.time.Clock()
        status = False
        while not status:
            status = self.handleEvents()
            clock.tick(10)
    
    def showAboutScreen(self):
        self.clearPlayArea()
        center = (self.screen.get_width()/2, self.screen.get_height()/2)
        self.screen.blit(self.banner, (160,center[1]-(self.banner.get_height()/2)))
        self.drawText("Reaction face-off", center,
                                  self.screen.get_height()-40, "right")
        self.drawMainText('''A fast-paced 2-player game that puts your reflexes, knowledge and brainpower to the test. Written in python using pygame, released under the GPLv3. \n (C) 2010 Carol Alexandru \n <sphaerodactylus@gmail.com>''',
        single=2, size=28)
        self.button1.draw()
        self.button2.draw()
        #~ self.display.blit(self.screen, (0,0))
        pygame.display.flip()
        clock = pygame.time.Clock()
        status = False
        while not status:
            status = self.handleEvents()
            clock.tick(10)
            
    def showHelpScreen(self):
        self.clearPlayArea()
        center = (self.screen.get_width()/2, self.screen.get_height()/2)
        self.drawText('''Round after round you compete in different games against your opponent. Every minigame makes a statement, and if it becomes true, you have to hit your button to gain a point. If you're wrong, your opponent gets a point.
        
        You can disable games you want to avoid from the games menu.
        
        Press P to pause, Q to quit and R to return to the main menu during a game.
        ''', center, self.screen.get_height()-40, "right", size=30)
        self.button1.draw()
        self.button2.draw()
        #~ self.display.blit(self.screen, (0,0))
        pygame.display.flip()
        clock = pygame.time.Clock()
        status = False
        while not status:
            status = self.handleEvents()
            clock.tick(10)
        
    def showGamesScreen(self):
        self.clearPlayArea()
        center = (self.screen.get_width()/2, self.screen.get_height()/2)
        slots = []
        rows = len(self.minigames)/2+1 # 1 for rounding, 1 for back button
        w = self.screen.get_width()
        h = self.screen.get_height()
        for x in range(0, rows):
            for y in range(0, 2):
                slots.append([x*(w/rows)+2,y*(h/2)+2])
        self.gameScreenButtons = []
        for i, game in enumerate(self.minigames):
            button = Button(self, slots[i][0],slots[i][1],
                           w/rows-4, h/2-4, "right")
            self.gameScreenButtons.append(button)
            if game not in self.undesiredGames:
                button.toggle()
            button.setCaption(self.minigames[i], textSize=30)
            button.draw()
        button = Button(self, slots[len(slots)-1][0],slots[len(slots)-1][1],
                           w/rows-4, h/2-4, "right")
        self.gameScreenButtons.append(button)
        button.setCaption("back", textSize=30)
        button.draw()
        #~ self.display.blit(self.screen, (0,0))
        pygame.display.flip()
        clock = pygame.time.Clock()
        status = False
        while not status:
            status = self.handleEvents()
            clock.tick(10)
            
    def handleGameScreenEvent(self, event, x, y):
        for i, button in enumerate(self.gameScreenButtons):
            if button.collidepoint(x,y):
                #back button
                if self.gameScreenButtons[len(self.gameScreenButtons)-1] == button:
                    self.gamesScreen = False
                    return True
                #all other buttons
                if button.toggled:
                    if not (len(self.minigames)-1 == len(self.undesiredGames)):
                        self.undesiredGames.append(self.minigames[i])
                        button.toggle()
                else:
                    self.undesiredGames.remove(self.minigames[i])
                    button.toggle()
                button.draw()
        pygame.display.flip()
        
    
    def intro(self, text):
        '''displays a Main Text with the rules of the next minigame'''
        self.drawMainText(text)
        self.button1.draw()
        self.button2.draw()
        self.drawScore()
        line = Rect(self.screen.get_width()/2-2, 0,
                    4, self.screen.get_height())
        pygame.draw.rect(self.screen, (255,255,255), line)
        #~ self.display.blit(self.screen, (0,0))
        pygame.display.flip()
        clock = pygame.time.Clock()
        for i in range(0,30):
            status = self.handleEvents()
            clock.tick(10)
            
    def finalizeGame(self):
        '''at the end of a game, display score and declare winner'''
        if self.points1 > self.points2:
            self.button1.setColor((0,255,0))
            self.button1.setCaption(
                         "You're the winner!", (0,0,0), 60)
            self.button2.setColor((255,0,0))
            self.button2.setCaption(
                         "You lost the game :'(", (0,0,0), 60)
            text = "Player 1 wins " + str(self.points1) + ":" + str(self.points2)
        elif self.points2 > self.points1:
            self.button2.setColor((0,255,0))
            self.button2.setCaption(
                         "You're the winner!", (0,0,0), 60)
            self.button1.setColor((255,0,0))
            self.button1.setCaption(
                         "Better luck next time", (0,0,0), 60)
            text = "Player 2 wins " + str(self.points2) + ":" + str(self.points1)
        else:
            self.button2.setColor((255,165,0))
            self.button2.setCaption(
                         "You're on par", (0,0,0), 60)
            self.button1.setColor((255,165,0))
            self.button1.setCaption(
                         "You're on par", (0,0,0), 60)
            text = "You played a draw with a score of " + str(self.points1)
        line = Rect(self.screen.get_width()/2-2, 0,
                    4, self.screen.get_height())
        pygame.draw.rect(self.screen, (255,255,255), line)
        self.drawMainText(text)
        self.button1.draw()
        self.button2.draw()
        #~ self.display.blit(self.screen, (0,0))
        pygame.display.flip()
        clock = pygame.time.Clock()
        for i in range(0,80):
            status = self.handleEvents()
            clock.tick(10)
    
    def drawMainText(self, text, color=None, single=None, size=None):
        '''draws big text into the playing'''
        vCenter = self.screen.get_height()/2
        vTot = self.screen.get_height()-40
        hHalf = self.screen.get_width()/2
        lCenter = ((((hHalf)-150)/2)+150, vCenter)
        rCenter = ((hHalf)+(hHalf-150)/2, vCenter)
        if not single:
            self.drawText(text, lCenter, vTot, "left", color, size)
            self.drawText(text, rCenter, vTot, "right", color, size)
        elif single == 1:
            self.drawText(text, lCenter, vTot, "left", color, size)
        elif single == 2:
            self.drawText(text, rCenter, vTot, "right", color, size)
        
    
    def clearPlayArea(self):
        '''blanks the screen'''
        self.screen.fill((0,0,0))
    
    def drawScore(self):
        '''draws the small score numbers into the corners'''
        textSize = 40
        textFont = pygame.font.Font(None, textSize)
        w = self.screen.get_width()
        h = self.screen.get_height()
        text = textFont.render(str(self.points1), 1, (200,200,200))
        text = pygame.transform.rotate(text, -90)
        self.screen.blit(text, (165, 5))
        text = pygame.transform.rotate(text, 180)
        self.screen.blit(text, (w/2-textSize, h-5-text.get_height()))
        text = textFont.render(str(self.points2), 1, (200,200,200))
        text = pygame.transform.rotate(text, 90)
        self.screen.blit(text, (w-190,  h-5-text.get_height()))
        text = pygame.transform.rotate(text, 180)
        self.screen.blit(text, (w/2+9, 5))
    
    def drawText(self, text, center, width, orientation=None,
                 color=None, size=None):
        '''prints multiline text as a centered block'''
        if not orientation:
            orientation = "right"
        if not color: color = (200,200,200)
        if not size: size = 70
        gap = int(size/10)
        textFont = pygame.font.Font(None, size)
        wordsToWrite = text.rstrip().split(" ")
        currLine = ""
        numLines = 0
        maxWidthFound = 0
        lines = []
        # split text into chunks that fit on a line
        for n, word in enumerate(wordsToWrite):
            # check if newlines appear. If yes, start a new line
            newline = True
            while newline:
                newline = word.find("\n")
                if newline != -1:
                    newWord = word[0:newline]
                    word = word[newline:0]
                    wordsToWrite.insert(n+1, "<newline>")
                    wordsToWrite.insert(n+1, newWord)
                else:
                    newline = False
            if (word == "<newline>") or ((textFont.size(currLine + " " + word))[0] > width):
                if (textFont.size(currLine))[0] > maxWidthFound:
                    maxWidthFound = (textFont.size(currLine))[0]
                lines.append (textFont.render(
                              currLine.rstrip().lstrip(), 1, color))
                currLine = ""
                numLines = numLines + 1
                if (word != "<newline>"):
                    currLine = currLine + " " + word
            else:
                currLine = currLine + " " + word
        if currLine != "":
            lines.append (textFont.render(
                          currLine.rstrip().lstrip(), 1, color))
            currLine = ""
            numLines = numLines + 1
        #~ self.image = pygame.Surface((maxWidthFound + gap, 
                     #~ numLines * textFont.get_height() + gap))
        # blit the possibly rotated, line by line
        offset = 0
        numberOfLines = len(lines)
        if orientation == "left": degrees = -90
        if orientation == "right": degrees = 90
        if orientation == "down": degrees = 0
        if orientation == "up": degrees = 180
        for lineNum in range(numLines):
            text = lines[lineNum]
            text = pygame.transform.rotate(text, degrees)
            if orientation == "left":
                textpos = text.get_rect(centerx=center[0]+(
                            ((size+gap)*(numberOfLines-1))/2)-offset,
                            centery=center[1])
            if orientation == "right":
                textpos = text.get_rect(centerx=center[0]-(
                            ((size+gap)*(numberOfLines-1))/2)+offset,
                            centery=center[1])
            if orientation == "down":
                textpos = text.get_rect(centerx=center[0],
                            centery=center[1]-(
                            ((size+gap)*(numberOfLines))/2)+offset)
            if orientation == "up":
                textpos = text.get_rect(centerx=center[0],
                            centery=center[1]+(
                            ((size+gap)*(numberOfLines))/2)-offset)    
            self.screen.blit(text, textpos)
            offset = offset + size + gap
    
    def load_image(self, name, colorkey=None):
        '''loads images from the data directory'''
        fullname = os.path.join('data', name)
        try:
            image = pygame.image.load(fullname)
        except pygame.error, message:
            print 'Cannot load image:', name
            raise SystemExit, message
        image = image.convert()
        if colorkey is not None:
            if colorkey is -1:
                colorkey = image.get_at((0,0))
            image.set_colorkey(colorkey, RLEACCEL)
        return image, image.get_rect()
    
    def discardEvents(self):
        '''empties the event queue discarding all events in it'''
        for event in pygame.event.get():
            pass
    
    def adjustSizesToScreen(self, size):
        '''resizes and moves stuff to fit new screen size'''
        self.screen = pygame.display.set_mode(size, pygame.RESIZABLE)
        self.button1.height = self.button2.height = self.showIntrosB.height = size[1]
        self.button2.left = size[0]-150
        self.area1 = Rect(0, 0, size[0]/2, size[1])
        self.area2 = Rect(size[0]/2, 0, size[0]/2, size[1])
        w = self.screen.get_width()
        h = self.screen.get_height()
        self.showIntrosB.left = w-160-100-90
        ypos = -(4000 - h)
        self.numberOfGamesS.left = w-160-90
        self.numberOfGamesS.top = ypos+200
        self.helpB.top = h-60
        self.aboutB.top = h-60
    
    def handleEvents(self):
        for event in pygame.event.get():
            if event.type == QUIT:
                exit(0)
            elif event.type == VIDEORESIZE:
                self.adjustSizesToScreen(event.size)
            elif event.type == KEYDOWN:
                if (event.key == K_ESCAPE) or (event.key == K_q):
                    exit(0)
                    #~ return True
                if event.key == K_r:
                    if self.playing:
                        return "return"
                if event.key == K_p:
                    if self.playing:
                        return "pause"
            elif event.type == MOUSEBUTTONUP:
                self.numberOfGamesS.sliding = False
            elif event.type == MOUSEBUTTONDOWN:
                if self.greeting:
                    x, y = pygame.mouse.get_pos()
                    if self.aboutScreen:
                        self.aboutScreen = False
                        return True
                    elif self.helpScreen:
                        self.helpScreen = False
                        return True
                    elif self.gamesScreen:
                        return self.handleGameScreenEvent(event, x, y)
                    elif self.pauseScreen:
                        return True
                    else:    
                        if self.button1.collidepoint(x, y):
                            self.ready1 = True
                            self.button1.setColor((0,255,0))
                            self.button1.draw()
                        elif self.button2.collidepoint(x, y):
                            self.ready2 = True
                            self.button2.setColor((0,255,0))
                            self.button2.draw()
                        elif self.showIntrosB.collidepoint(x, y):
                            self.showIntrosB.toggle()
                        elif self.numberOfGamesS.collidepoint(x, y):
                            self.numberOfGamesS.toggleSlide((x,y))
                        elif self.quitB.collidepoint(x,y):
                            exit(0)
                        elif self.aboutB.collidepoint(x,y):
                            self.aboutScreen = True
                            self.showAboutScreen()
                        elif self.helpB.collidepoint(x,y):
                            self.helpScreen = True
                            self.showHelpScreen()
                        elif self.gamesB.collidepoint(x,y):
                            self.gamesScreen = True
                            self.showGamesScreen()
                elif self.playing:
                    x, y = pygame.mouse.get_pos()
                    if self.area1.collidepoint(x, y):
                        return self.g.evaluate(1)
                    elif self.area2.collidepoint(x, y):
                        return self.g.evaluate(2)
                else:
                    pass
class Button(Rect):
    def __init__(self, reactor, x, y, w, h, orientation, color=None):
        Rect.__init__(self, x, y, w, h)
        self.center = (self.left+self.width/2, self.top+self.height/2)
        self.reactor = reactor
        self.orientation = orientation
        if not color: self.color = (0, 71, 171)
        else: self.color = color
        self.caption = False
        self.textSize = 40
        self.textColor = (255,255,255)
        self.toggled = False
        self.glowing = True
        self.origColor = self.color
        self.v1 = 0
        self.v2 = 0
        self.v3 = 0
    
    def toggle(self):
        if self.toggled:
            self.toggled = False
            self.textColor = (255,255,255)
        else:
            self.toggled = True
            self.textColor = (0,255,0)
    
    def setColor(self, color):
        self.color = color
        
    def setCaption(self, text, textColor=None, textSize=None):
        self.caption = text
        if textColor: self.textColor = textColor
        if textSize: self.textSize = textSize
    
    def draw(self):
        pygame.draw.rect(self.reactor.screen, self.color, self)
        if self.caption:
            self.reactor.drawText(self.caption, self.center,
                                  self.height-40, self.orientation,
                                  self.textColor, self.textSize)
    
    def glow(self):
        diff = 0
        for i in range(0,3):
            diff += self.origColor[i]-self.color[i]
        if self.glowing == True:
            if diff > 60:
                self.glowing = False
            else:
                c1 = self.color[1]-3
                c2 = self.color[2]-3
                self.color = (self.color[0],c1,c2)
        if self.glowing == False:
            if diff < -60:
                self.glowing = True
            else:
                c1 = self.color[1]+3
                c2 = self.color[2]+3
                self.color = (self.color[0],c1,c2)
                #~ self.color = [i + 2 for i in self.color]

class Slider(Button):
    def __init__(self, reactor, x, y, w, h, orientation, color=None):
        Button.__init__(self, reactor, x, y, w, h, orientation, color)
        self.value = 15
        self.pos = 0
        self.sliding = False
        self.setCaption("lol")
        
    def toggleSlide(self, coords):
        if self.sliding:
            self.sliding = False
        else:
            self.sliding = True
        if (self.orientation == "right") or (self.orientation == "left"):
            self.pos = coords[1]-self.top
        if (self.orientation == "up") or (self.orientation == "down"):
            self.pos = coords[0]-self.left
    
    def slide(self):
        if self.sliding:
            coords = pygame.mouse.get_pos()
            if (self.orientation == "right") or (self.orientation == "left"):
                newTop = coords[1]-self.pos
                if newTop > 0:
                    self.top = 0
                elif newTop < self.reactor.screen.get_height()-self.height:
                    self.top = self.reactor.screen.get_height()-self.height
                else:
                    self.top = coords[1]-self.pos
                    self.value = ((self.bottom-self.reactor.screen.get_height()) / 20)
                    if not (self.value % 2): self.value += 1
            if (self.orientation == "up") or (self.orientation == "down"):
                self.left = coords[0]-self.pos
    
    def draw(self):
        pygame.draw.rect(self.reactor.screen, self.color, self)
        h = self.reactor.screen.get_height()
        center = (self.left+(self.width/2-15), h/2)
        if not self.caption: string = str(self.value)
        else: string = self.caption + ": " + str(self.value)
        self.reactor.drawText(string, center,
                              h-40, self.orientation,
                              self.textColor, self.textSize)
        line = Rect(self.left+self.width-20, self.top+20,
                    5, self.height-40)
        pygame.draw.rect(self.reactor.screen, (255,255,255), line)
        for i in range (20,self.height-20, (self.height-40)/19+1):
            mark = Rect(self.left + self.width-35, self.top+i,
                        15, 5)
            pygame.draw.rect(self.reactor.screen, (255,255,255), mark)

class Starfield(object):
    def __init__(self, reactor, x, y, w, h, numberOfStars, direction):
        self.reactor = reactor
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.direction = direction
        self.stars = []
        for n in range(0,numberOfStars):
            star = [choice(xrange(w))+x,choice(xrange(h))+y,choice(range(2,5))]
            self.stars.append(star)
    
    def update(self):
        for s in self.stars:
            if self.direction == "up":
                newPos = s[1]+s[2]
                if newPos < self.y:
                    s[1] = self.y+self.h
                else:
                    s[1] = s[1]-s[2]
            if self.direction == "down":
                newPos = s[1]-s[2]
                if newPos > self.y+self.h:
                    s[1] = self.y
                else:
                    s[1] = s[1]+s[2]
            if self.direction == "left":
                newPos = s[0]-s[2]
                if newPos < self.x:
                    s[0] = self.x+self.w
                else:
                    s[0] = s[0]-s[2]
            if self.direction == "right":
                newPos = s[0]+s[2]
                if newPos > self.x+self.w:
                    s[0] = self.x
                else:
                    s[0] = s[0]+s[2]
            star = Rect(s[0], s[1],
                    s[2], s[2])
            pygame.draw.rect(self.reactor.screen, (255,255,255), star)

# Start the game
ReactionFaceoff()











